﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.SymbolicExpressionTreeEncoding {
  public sealed class GroupSymbol : Symbol, IReadOnlySymbol {
    private List<ISymbol> symbols;

    [Storable]
    public IEnumerable<ISymbol> Symbols {
      get { return symbols; }
      private set { symbols = new List<ISymbol>(value); }
    }

    [StorableConstructor]
    private GroupSymbol(bool deserializing) : base(deserializing) { }
    private GroupSymbol(GroupSymbol original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GroupSymbol(this, cloner);
    }

    public GroupSymbol(string name) : this(name, "A symbol to group other symbols.") { }
    public GroupSymbol(string name, string description) : this(name, description, Enumerable.Empty<ISymbol>()) { }
    public GroupSymbol(string name, string description, IEnumerable<ISymbol> symbols)
      : base(name, description) {
      this.symbols = new List<ISymbol>(symbols);
    }
  }
}
